package db;

import java.util.*;
import java.sql.*;

class DbConnection 
{
    private final Connection m_jdbcConn;
    private final Hashtable  m_psTab = new Hashtable();

    private DbConnection(Connection c)
    {
	m_jdbcConn = c;
    }

    public final Connection getJdbcConnection()
    {
	return m_jdbcConn;
    }

    /*
     *  Get a prepared statement
     */
    public final PreparedStatement getStatement(
	    final String name,			       
	    final String sql)
    {
	PreparedStatement ps = null;
	synchronized (m_psTab) {
	    ps = (PreparedStatement) m_psTab.get(name);
	    if (ps == null) {
		try {
		    ps = m_jdbcConn.prepareStatement(sql);
		    m_psTab.put(name, ps);
		}
		catch (SQLException e) {
		    throw new DbException(e);
		}
	    }
	}
	return ps;
    }


    //	Class variable: Pool of DbConnection
    private static Stack s_pool;

    public static DbConnection pop()
    {
	synchronized (s_pool) {
	    while (s_pool.empty()) {
		try {
		    s_pool.wait();
		}
		catch (InterruptedException e) {
		    ;
		}
	    }
	    return (DbConnection) s_pool.pop();
	}
    }

    public static void push(DbConnection c)
    {
	try {
	    c.m_jdbcConn.setAutoCommit(false);
	}
	catch (SQLException e) {
	    System.err.println("unexpected error: " + e);
	    e.printStackTrace();
	    System.exit(44);
	}

	synchronized (s_pool) {
	    s_pool.push(c);
	    s_pool.notify();
	}
    }

	

    /*
     *  Initialization. Sets up maxConnections
     *  to the database in a pool for later use.
     */
    public static synchronized void init(int maxConnections,
					 String user,
					 String password,
					 String url,
					 String driver)
    {
	if (s_pool != null) {
	    throw new 
		RuntimeException("already initialized");
	}
		      
	
	// load the driver.
	try {
	    Class.forName(driver);
	}
	catch (ClassNotFoundException e) {
	    throw new RuntimeException("error loading driver - " 
				       + driver + ": " + e);
	}
     
	s_pool = new Stack();

	try {
	    for (int i = 0; i < maxConnections; i++)  {
		final Connection c = DriverManager.getConnection(
						url, user, password);
		    c.setAutoCommit(false);
		    s_pool.push(new DbConnection(c));
		}
	}
	catch (Exception e) {
	    System.err.println("cannot connect to db: " + e);
	    e.printStackTrace();
	    System.exit(44);
	}
    }

}
