package db;

 import java.sql.*;
 import java.io.*;


/*
 *  Base class of all select statements
 */
public abstract class DbQuery {

    private DbTrans           m_trans = null;
    private String            m_name = null;
    private PreparedStatement m_ps = null;
    private ResultSet         m_res = null;
    private String            m_sql = null;
    private Statistic         m_stat = null;
    
    protected DbQuery(final DbTrans trans, 
		      final String name,
		      final String sql, 
		      final Statistic stat)
    {
        m_trans = trans;
        m_sql = sql;
	m_name = name;
        m_ps = m_trans.getStatement(name, sql);
	m_stat = stat;
    }

    /*
     *  Fill up parameters for the jdbc prepared statement.
     */
    protected abstract void fillParam(PreparedStatement ps)
        throws SQLException;

    /*
     *  Fill the result object using the jdbc results set.
     */
    protected abstract Object fillResult(final ResultSet rs)
	throws SQLException;

    /*
     *  Get the next result object
     */
    protected final Object next() 
    {
	try {
	    if (null != m_res && !m_res.next())  {
		m_res.close();
		m_res = null;
	    }
	    return fillResult(m_res);
	}
	catch (SQLException e) {
	    throw new DbException(e);
	}
    }

    /*
     *  Execute the select statement.
     */
    public final void execute() 
    {
	long t0 = System.currentTimeMillis();
        //  set parameters and do it!
	try {
	    fillParam(m_ps);
	    m_res = m_ps.executeQuery();
	}
	catch (SQLException e) {
	    throw new DbException(e);
	}
	finally {
	    long t1 = System.currentTimeMillis();
	    m_stat.add((int) (t1 - t0));
	}
    }

    /*
     *  Execute the update stmt.
     */
    protected final int executeUpdate()
    {
	long t0 = System.currentTimeMillis();
        //  set parameters and do it!
	try {
	    fillParam(m_ps);
	    return m_ps.executeUpdate();
	}
	catch (SQLException e) {
	    throw new DbException(e);
	}
	finally {
	    long t1 = System.currentTimeMillis();
	    m_stat.add((int)(t1 - t0));
	}
    }


    /*
     *  Close the select statement.
     */
    public final void close() 
    {
	if (null == m_ps)
	    return;

	try {
	    //  clear parameters on the statement
	    try {
		m_ps.clearParameters();
	    }
	    catch (SQLException e) {
		;
	    }


	    //  close out result set if any
	    if (null != m_res) {
		try {
		    m_res.close();
		}
		catch (SQLException e) {
		    ;
		}
	    }
	}
	finally {
	    m_trans = null;
	    m_ps = null;
	    m_res = null;
	    m_name = null;
	}
    }



    /*
     *  Catch any leak in the system
     */
    protected void finalize() 
    		throws Throwable
    {
	if (null != m_ps) {
	    System.err.println("sql stmt leak detected");
            close();
        }
        super.finalize();
    }

}
